import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { lastValueFrom } from 'rxjs';
import { Structure } from '../../../../../models/structure.model';
import { GeojsonService } from '../../../../../services/geojson.service';
import { NotificationService } from '../../../../../services/notification.service';
import { RdvsService } from '../../../../../services/rdvs.service';
import { StructureService } from '../../../../../services/structure.service';
import { Filter } from '../../../../../structure-list/models/filter.model';

@Component({
  selector: 'app-meeting-rdvs-doing',
  templateUrl: './meeting-rdvs-doing.component.html',
  styleUrls: ['./meeting-rdvs-doing.component.scss'],
})
export class MeetingRdvsDoingComponent implements OnInit {
  @Input() form: UntypedFormGroup;
  public structureList: Structure[] = [];
  public filters: Filter[] = [];
  public shouldReset = false;
  @Output() setResetOrientation = new EventEmitter();
  constructor(
    private structureService: StructureService,
    private rdvsService: RdvsService,
    private notificationService: NotificationService,
    private geoJsonService: GeojsonService
  ) {}

  async ngOnInit(): Promise<void> {
    try {
      this.buildFilters();
      await this.getData();
  
      const addressCoords = this.form.get('address')?.value?.coordinates;
  
      if (addressCoords) {
        this.structureList = this.sortByDistance(this.structureList, addressCoords);
      }
  
      if (this.structureList.length === 0) {
        this.shouldReset = true;
        this.setResetOrientation.emit();
        return;
      }
  
      this.rdvsService.sendStructureArray(this.structureList);
      this.setResetOrientation.emit();
    } catch (error) {
      console.error(error);
      this.notificationService.showError('Une erreur est survenue', error.message);
    }
  }

  private async getData(): Promise<void> {
    try {
      // Get structures based on the filters
      const structures = await lastValueFrom(this.structureService.getStructures(this.filters));

      // Filter out structures that have no idCNFS or have a duplicate idCNFS
      this.structureList = structures.reduce((acc, curr) => {
        if (!curr.idCNFS || acc.some(structure => structure.idCNFS === curr.idCNFS)) {
          // If current structure has no idCNFS or a duplicate idCNFS, skip it
          return acc;
        }
        // If current structure has a unique idCNFS, add it to the list
        return [...acc, curr];
      }, []);
    } catch (error) {
      console.error(error);
      this.notificationService.showError('Error fetching structures', error.message);
    }
  } 
  
  private buildFilters(): void {
    const filters = [];
    for (let [categ, moduleIdArray] of Object.entries(this.form.get('categories').value)) {
      const idArray = moduleIdArray as string[];
      if (idArray.length)
        idArray.forEach((filterId: string) => {
          filters.push(new Filter(categ, filterId));
        });
    }
    if (this.form.get('pmrAccess').value) filters.push(new Filter('pmrAccess', this.form.get('pmrAccess').value));
    if (this.form.get('digitalPass').value) filters.push(new Filter('labelsQualifications', 'passNumerique'));
    filters.push(...this.form.get('onlineDemarcheType').value);
    this.filters = filters;
  }
  private sortByDistance(structures: Structure[], addresscoords: number[]): Structure[] {
    return structures.sort(
      (a, b) =>
        parseInt(this.geoJsonService.getDistance(a.getLat(), a.getLon(), addresscoords[1], addresscoords[0], 'M')) -
        parseInt(this.geoJsonService.getDistance(b.getLat(), b.getLon(), addresscoords[1], addresscoords[0], 'M'))
    );
  }
}
